import stat
import string

template = """#!${python_path}
# VAM-LAUNCHER-SCRIPT: '${package}==${version}','${entry_point_group}','${entry_point}'
__requires__ = '${package}==${version}'
import sys
from pkg_resources import load_entry_point

if __name__ == '__main__':
    sys.exit(
        load_entry_point('${package}==${version}', '${entry_point_group}', '${entry_point}')()
    )
"""


def create_launcher_text(**kwargs):
    t = string.Template(template)
    return t.safe_substitute(kwargs)


def install_launcher(path, text):
    with open(str(path), 'w+') as f:
        f.write(text)
    path.chmod(path.stat().st_mode | stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH)



import pathlib

import pkg_resources

from .environment import vam_dir


def package_dir(package: str) -> pathlib.Path:
    return vam_dir() / package


def get_distribution(package: str) -> pkg_resources.Distribution:
    lib = package_dir(package) / 'lib'
    package_dists = []
    for py_version in lib.iterdir():  # Iterate over 'python3.4' etc.
        if not py_version.is_dir():
            continue
        site_packages = py_version / 'site-packages'
        dists = [d for d in pkg_resources.find_distributions(str(site_packages))]
        package_dists += [d for d in dists if d.project_name == package]

    if not package_dists:
        raise Exception('Distribution not found in {}'.format(lib))

    return package_dists[0]


def get_entry_points(package: str):
    dist = get_distribution(package)
    entry_map = pkg_resources.get_entry_map(dist)
    for entry_point_group, entry_points in entry_map.items():
        for entry_point in entry_points.values():
            yield entry_point_group, entry_point



from distutils.core import setup
from setuptools import find_packages

from vam import __version__


def readme():
    with open('README.rst') as f:
        return f.read()


setup(
    name='vam',
    version=__version__,
    url='https://github.com/milliams/vam',
    long_description=readme(),
    packages=find_packages(exclude=['doc', 'tests']),
    license='MIT',
    author='Matt Williams',
    author_email='matt@milliams.com',
    description='Virtualenv Application Manager',
    entry_points={
        'console_scripts': [
            'vam=vam.vam:vam',
        ],
    },
    install_requires=[
        'click'
    ],
)



import os
import pathlib


def home() -> pathlib.Path:
    return pathlib.Path(os.path.expanduser('~'))


def vam_dir() -> pathlib.Path:
    return home() / '.vam'


def packages():
    return [x for x in vam_dir().iterdir() if x.is_dir()]




import pathlib
import shutil
import subprocess
import venv

import click

from .environment import home, packages
from .launcher import create_launcher_text, install_launcher
from .package import package_dir, get_entry_points, get_distribution


def pip_command(package: str) -> pathlib.Path:
    return package_dir(package) / 'bin' / 'pip'


def call_pip(package: str, args):
    pip = pip_command(package)
    return subprocess.check_call([str(pip)] + args, stdout=subprocess.PIPE)


config = {
    'bindir': home() / '.local' / 'bin'
}


@click.group()
def vam():
    """Virtualenv Application Manager AKA 'pia'"""
    pass


@vam.command()
@click.argument('package')
@click.option('--upgrade/--no-upgrade', default=False)
def install(package: str, upgrade: bool):
    """Install a package"""
    click.echo('Installing {}'.format(package))

    if package_dir(package).exists() and not upgrade:
        click.echo('Application {} already installed, to upgrade use --upgrade'.format(package))
        exit(1)

    # Create venv
    venv_dir = package_dir(package)
    venv.create(str(venv_dir), clear=True, with_pip=True)

    # Ask pip to install the package
    call_pip(package, ['install', package])

    # Install entry-point launchers
    for entry_point_group, entry_point in get_entry_points(package):
        python_path = package_dir(package) / 'bin' / 'python'  # TODO ask setuptools for this info?
        launcher = create_launcher_text(
            package=package,
            version=entry_point.dist.version,
            entry_point=entry_point.name,
            entry_point_group=entry_point_group,
            python_path=str(python_path)
        )

        launcher_path = config['bindir'] / entry_point.name
        install_launcher(launcher_path, launcher)


@vam.command()
@click.argument('package')
def remove(package):
    """Remove a package"""

    for entry_point_group, entry_point in get_entry_points(package):
        launcher_path = config['bindir'] / entry_point.name
        launcher_path.unlink()

    d = package_dir(package)
    shutil.rmtree(str(d))


@vam.command()
@click.argument('package')
def info(package):
    """Display information about a package"""

    dist = get_distribution(package)
    click.echo('{}=={}'.format(package, dist.version))
    for entry_point_group, entry_point in get_entry_points(package):
        click.echo('  {}'.format(entry_point_group))
        click.echo('    {}'.format(entry_point.name))


@vam.command('list')
def _list():
    """List all installed packages"""
    for p in packages():
        package = p.name
        dist = get_distribution(package)
        click.echo('{}=={}'.format(package, dist.version))
